<?PHP
/*******************************************************************
 *  cmsWorks                                                       *
 *  Copyright (C) 2003-2005 Ryan Skov                              *
 *                                                                 *
 *  Licensed under the terms of the GNU General Public License:    *
 * 		http://www.opensource.org/licenses/gpl-license.php           *
 *                                                                 *
 *  For further information visit:                                 *
 * 		http://www.netslave.dk/cmsworks                              *
 *                                                                 *
 *  Version:  1.0.1                                               *
 *	Modified: 2006-02-05 22:18                                     *
 *                                                                 *
 *  File Authors:                                                  *
 *    Ryan Skov (ryan[at]netslave.dk)                              *
 *******************************************************************/

class GALLERY
{
	var $DEBUG = false;
	
	var $FILE_TYPES = array("png", "jpg", "jpeg");
	
	// gallery basedir
	var $base_dir = "../../files/gallery";
	// holds all the dir paths
	var $dir_array = array(".");
	// holds pics that is missing thumbs
	var $pic_array = array();
	
	// this function scans a given basedir for dirs and adds them to the global $dir_array var
	// it is recursive so it takes all sub dirs, exept dirs named thumbs 
	function GALLERY_scan_for_dirs($base_dir)
	{
		if(is_dir($base_dir))
		{
			if($handle = opendir($base_dir)) 
			{
				while(false !== ($file = readdir($handle))) 
				{
					if ($file != "." && $file != ".." && $file != "thumbs") 
					{
						if((fileperms("$base_dir/$file") & 0x4000) == 0x4000)
						{
							// add dir to array, with full path
							array_push($this->dir_array, str_replace($this->base_dir."/", "", "$base_dir/$file"));
							
							// call this function recursive to add the dirs in this dir to array
							$this->GALLERY_scan_for_dirs("$base_dir/$file");
							
							if($this->DEBUG)
								echo "DIR: ";
						}
						else
						{
							if($this->DEBUG)
								echo "FILE: ";
						}
						if($this->DEBUG)
			   			echo "$file<br>";
					}
				}
				closedir($handle);
			}
		}
	}
	
	// checks to se if all thumbs is generated in a dir
	function GALLERY_check_dir($base_dir, $return_type)
	{
		// array for all pics in dir
		$pic_array = array();
		// create array for pictures that are not ok
		$pic_missing_array = array();
		// dir status
		$dir_status = true;
		
		if(is_dir($base_dir))
		{
			if($handle = opendir($base_dir)) 
			{
				while(false !== ($file = readdir($handle))) 
				{
					if ($file != "." && $file != ".." && $file != "thumbs") 
					{
						// find extension
						$ext = explode(".", $file);
						
						// check to se if it is a dir. The is_dir functions is broken in safe_mode
						if(!(fileperms("$base_dir/$file") & 0x4000) == 0x4000 AND in_array(strtolower($ext[(count($ext)-1)]), $this->FILE_TYPES))
						{
							if($this->DEBUG)
								echo "FILE: $base_dir/$file<br>";
							
							// add image to pic array
							array_push($pic_array, $file);	
								
							// check to se if the two thums is present
							if(!is_file("$base_dir/thumbs/d-$file"))
							{
								if($this->DEBUG)
									echo "-- THUMB d missing: $base_dir/thumbs/d-$file<br>";
								
								// if the file is not in the array then add it									
								if(!in_array($file, $pic_missing_array))
									array_push($pic_missing_array, $file);
									
								// dir status
								$dir_status = false;
							}
							
							if(!is_file("$base_dir/thumbs/p-$file"))
							{
								if($this->DEBUG)
									echo "-- THUMB p missing: $base_dir/thumbs/p-$file<br>";
									
								// if the file is not in the array then add it
								if(!in_array($file, $pic_missing_array))
									array_push($pic_missing_array, $file);
	
								// dir status
								$dir_status = false;
							}			 
						}
					}
				}
				closedir($handle);
			}
		}
				
		if($return_type == "STATUS")
			return $dir_status;
		else if($return_type == "ALL")
			return $pic_array;
		else
			return $pic_missing_array;
	}
	
	
	// generates thumbs of pictures
	function GALLERY_generate_thumbs($base_dir, $size, $logo)
	{	
		// if there is somthing wrong
		if(!$this->GALLERY_check_dir("$this->base_dir/$base_dir", "STATUS"))
		{
			if($this->DEBUG)
				echo "Dir ($base_dir) is not ok<br>";
				
			// get list of pics to generate thumbs for
			$pic_array = $this->GALLERY_check_dir("$this->base_dir/$base_dir", "PICS");
			
			// make shure the thumb dir is there
#			if(!(fileperms("../../$base_dir/thumbs") & 0x4000) == 0x4000)
			if(!is_dir("$this->base_dir/$base_dir/thumbs"))
				mkdir("$this->base_dir/$base_dir/thumbs");

			// run tru pic array and generate the thumbs	
			foreach ($pic_array as $value)	
			{
				// in file name
				$file_in = "$this->base_dir/$base_dir/$value";
				
				if($this->DEBUG)
					echo $file_in."<br>";
					
				if(!is_file("$this->base_dir/$base_dir/thumbs/d-$value"))
					$this->GALLERY_resize_img($file_in, "$this->base_dir/$base_dir/thumbs/d-$value", $size["dir-height"], $size["dir-width"], $logo);
				
				if(!is_file("$this->base_dir/$base_dir/thumbs/p-$value"))
					$this->GALLERY_resize_img($file_in, "$this->base_dir/$base_dir/thumbs/p-$value", $size["pic-height"], $size["pic-width"], $logo);					
			}
		}
	}
	
	
	// generates a thumbnail with a logo if specified
	function GALLERY_resize_img($file_in, $file_out, $height, $width, $logo)
	{
		// include image toolbox
		if(is_file("../../classes/imageToolbox/Image_Toolbox.class.php"))
			include_once("../../classes/imageToolbox/Image_Toolbox.class.php");
		else if(is_file("classes/imageToolbox/Image_Toolbox.class.php"))
			include_once("classes/imageToolbox/Image_Toolbox.class.php");
		else
		{
			syslog("ERROR: could not include Image Toolbox class (class.gallery.php)");
			return;
		}

		// img class
		$img = new Image_Toolbox("$file_in");

		// add logo if one is defined in the config
		if(!empty($logo["file"]) AND is_file($logo["file"]))
		{
			$img->addImage($logo["file"]);
	
			switch($logo["location"])
			{
				case "RIGHT_TOP":
					$img->blend('right','top');
					break;
				case "LEFT_BOTTOM":
					$img->blend('left','bottom');
					break;
				case "LEFT_TOP":
					$img->blend('left','top');
					break;
				default:
					$img->blend('right','bottom');
					break;
			}
		}

		// get image size
		$s_height = $height;
		$s_width = $width;
		list($c_width, $c_height, $type, $attr) = getimagesize("$file_in");
		if($c_width > $width OR empty($height) OR $height == 0)
		{
			$f = $width/$c_width;
			$s_height = $c_height*$f;
		}
		if(($c_height > $height AND !empty($height) AND $height != 0) OR empty($width) OR $width == 0)
		{
			$f = $height/$c_height;
			$s_width = $c_width*$f;
			$s_height = $height;
		}
		// set best quality resize method, this is slover :-/
		$img->setResizeMethod('resample');
		settype($s_width, "integer");
		settype($s_height, "integer");
		$img->newOutputSize($s_width, $s_height, false, false);
		$img->save("$file_out");
	}
}

/*
// test
$g = new GALLERY();
#$g->DEBUG = true;
$g->GALLERY_scan_for_dirs($g->base_dir);
print_r($g->dir_array);
echo "<br>";

foreach ($g->dir_array as $value) 
{
	echo "DIR = ".$value." = ".$g->GALLERY_check_dir("$g->base_dir/$value", "STATUS")." FILES: ";
	$ged = $g->GALLERY_check_dir("$g->base_dir/$value", "PICS");
	print_r($ged);	
	echo "<br>";
}

echo "<br><br>Generate<br>";
$size["dir-width"] = 100;
$size["dir-height"] = 80;
$size["pic-width"] = 100;
$size["pic-height"] = 80;
#$g->DEBUG = true;
foreach ($g->dir_array as $value)
{
	$g->GALLERY_generate_thumbs($value, $size, "");
	echo "<br>";	
}	

*/
?>